use super::{
    CommentCreated, CommentDeleted, CommentUpdated, RemoteCommentCreated, RemoteCommentUpdated,
};
use crate::{store::Comment, Completed, Context, Error, Required};
use activitystreams::{
    activity::{Announce, Create, Delete, Update},
    base::AnyBase,
    context,
    object::Note,
    prelude::*,
    public, security,
};
use url::Url;

fn build_comment(
    comment: Comment,
    note_id: Url,
    person_id: Url,
    ctx: &Context,
) -> Result<AnyBase, Error> {
    let published = comment.published();
    let endpoints = ctx
        .apub
        .endpoints_for_profile(comment.profile_id())?
        .req()?;

    let mut note = Note::new();
    note.set_id(note_id)
        .set_content(comment.body())
        .set_published(published.into())
        .set_attributed_to(person_id)
        .add_to(endpoints.followers)
        .add_cc(public());
    if let Some(comment_id) = comment.comment_id() {
        let in_reply_to = ctx.apub.apub_for_comment(comment_id)?.req()?;
        note.set_in_reply_to(in_reply_to);
    }

    let any_base = note.into_any_base()?;
    ctx.apub.store_object(&any_base)?;

    Ok(any_base)
}

impl Completed for CommentCreated {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let comment = ctx.store.comments.by_id(self.comment_id)?.req()?;
        let person_id = ctx.apub.apub_for_profile(comment.profile_id())?.req()?;
        let note_id = ctx.apub.info.gen_id();
        ctx.apub.comment(&note_id, self.comment_id)?;
        let note = build_comment(comment, note_id, person_id.clone(), ctx)?;

        let mut create = Create::new(person_id, note);
        create
            .set_id(ctx.apub.info.gen_id())
            .add_context(context())
            .add_context(security());
        let any_base = create.into_any_base()?;
        ctx.apub.store_object(&any_base)?;

        Ok(any_base)
    }
}

impl Completed for RemoteCommentCreated {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let comment = ctx.apub.object(&self.note_apub_id)?.req()?;
        let person_id = ctx.apub.apub_for_profile(self.profile_id)?.req()?;

        let mut announce = Announce::new(person_id, comment);
        announce
            .set_id(ctx.apub.info.gen_id())
            .add_context(context())
            .add_context(security());
        let announce = announce.into_any_base()?;
        ctx.apub.store_object(&announce)?;

        Ok(announce)
    }
}

impl Completed for CommentUpdated {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let comment = ctx.store.comments.by_id(self.comment_id)?.req()?;
        let person_id = ctx.apub.apub_for_profile(comment.profile_id())?.req()?;
        let note_id = ctx.apub.apub_for_comment(comment.id())?.req()?;
        let note = build_comment(comment, note_id, person_id.clone(), ctx)?;

        let mut update = Update::new(person_id, note);
        update
            .set_id(ctx.apub.info.gen_id())
            .add_context(context())
            .add_context(security());
        let any_base = update.into_any_base()?;
        ctx.apub.store_object(&any_base)?;

        Ok(any_base)
    }
}

impl Completed for RemoteCommentUpdated {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let person_id = ctx.apub.apub_for_profile(self.profile_id)?.req()?;
        let update = ctx.apub.object(&self.update_apub_id)?.req()?;

        let mut announce = Announce::new(person_id, update);
        announce
            .set_id(ctx.apub.info.gen_id())
            .add_context(context())
            .add_context(security());
        let announce = announce.into_any_base()?;
        ctx.apub.store_object(&announce)?;

        Ok(announce)
    }
}

impl Completed for CommentDeleted {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let person_id = ctx.apub.apub_for_profile(self.profile_id)?.req()?;
        let note = ctx.apub.object(&self.note_apub_id)?.req()?;

        let mut delete = Delete::new(person_id, note);
        delete
            .set_id(ctx.apub.info.gen_id())
            .add_context(context())
            .add_context(security());
        let any_base = delete.into_any_base()?;
        ctx.apub.store_object(&any_base)?;

        Ok(any_base)
    }
}
